@@ -31,7 +31,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)conf.h	8.264 (Berkeley) 9/26/96
+ *	@(#)conf.h	8.272 (Berkeley) 11/16/96
  */
 
 /*
@@ -41,7 +41,9 @@
 **	included in the next release.
 */
 
+#ifdef __GNUC__
 struct rusage;	/* forward declaration to get gcc to shut up in wait.h */
+#endif
 
 # include <sys/param.h>
 # include <sys/types.h>
@@ -210,6 +212,20 @@ extern void	hard_syslog(int, char *, ...);
 #endif
 
 
+/*
+**  IBM AIX 4.x
+*/
+
+#ifdef _AIX4
+# define _AIX3		1	/* pull in AIX3 stuff */
+# define USESETEUID	1	/* seteuid(2) works */
+# define TZ_TYPE	TZ_NAME	/* use tzname[] vector */
+# if _AIX4 >= 40200
+#  define HASSETREUID	1	/* setreuid(2) works as of AIX 4.2 */
+# endif
+#endif
+
+
 /*
 **  IBM AIX 3.x -- actually tested for 3.2.3
 */
@@ -226,6 +242,7 @@ extern void	hard_syslog(int, char *, ...);
 # define SFS_TYPE	SFS_STATFS	/* use <sys/statfs.h> statfs() impl */
 # define SPT_PADCHAR	'\0'	/* pad process title with nulls */
 # define LA_TYPE	LA_INT
+# define FSHIFT		16
 # define LA_AVENRUN	"avenrun"
 #endif
 
@@ -398,6 +415,12 @@ typedef int		pid_t;
 #    define snprintf	__snprintf	/* but names it oddly in 2.5 */
 #    define vsnprintf	__vsnprintf
 #   endif
+#   ifndef LA_TYPE
+#    define LA_TYPE	LA_KSTAT	/* use kstat(3k) -- may work in < 2.5 */
+#   endif
+#  endif
+#  ifndef HASGETUSERSHELL
+#   define HASGETUSERSHELL 0	/* getusershell(3) causes core dumps */
 #  endif
 
 # else
@@ -574,6 +597,7 @@ extern long	dgux_inet_addr();
 # define WAITUNION	1	/* use "union wait" as wait argument type */
 # define UID_T		int	/* compiler gripes on uid_t */
 # define GID_T		int	/* ditto for gid_t */
+# define MODE_T		int	/* and mode_t */
 # define sleep		sleepX
 # define setpgid	setpgrp
 # ifndef LA_TYPE
@@ -682,7 +706,6 @@ typedef int		pid_t;
 # if defined(__NetBSD__) && (NetBSD > 199307 || NetBSD0_9 > 1)
 #  undef SPT_TYPE
 #  define SPT_TYPE	SPT_BUILTIN	/* setproctitle is in libc */
-#  define setreuid	__setreuid
 # endif
 # if defined(__FreeBSD__)
 #  undef SPT_TYPE
@@ -778,31 +801,43 @@ extern int		errno;
 **	The third is for SCO UNIX 3.2v4.0/Open Desktop 2.0 and earlier.
 */
 
+/* SCO OpenServer 5 */
 #if _SCO_DS >= 1
 # include <paths.h>
 # define _SCO_unix_4_2
-# define HASSNPRINTF	1	/* has snprintf() call */
-# define HASFCHMOD	1	/* has fchmod() call */
-# define HASSETRLIMIT	1	/* has setrlimit() call */
+# define HASSNPRINTF	1	/* has snprintf(3) call */
+# define HASFCHMOD	1	/* has fchmod(2) call */
+# define HASSETRLIMIT	1	/* has setrlimit(2) call */
+# define USESETEUID	1	/* has seteuid(2) call */
+# define HASINITGROUPS	1	/* has initgroups(3) call */
+# define HASGETDTABLESIZE 1	/* has getdtablesize(2) call */
 # define RLIMIT_NEEDS_SYS_TIME_H	1
+# ifndef LA_TYPE
+#  define LA_TYPE	LA_DEVSHORT
+# endif
+# define _PATH_AVENRUN	"/dev/table/avenrun"
 #endif
 
-
+/* SCO UNIX 3.2v4.2/Open Desktop 3.0 */
 #ifdef _SCO_unix_4_2
 # define _SCO_unix_
 # define HASSETREUID	1	/* has setreuid(2) call */
 #endif
 
+/* SCO UNIX 3.2v4.0 Open Desktop 2.0 and earlier */
 #ifdef _SCO_unix_
 # include <sys/stream.h>	/* needed for IP_SRCROUTE */
 # define SYSTEM5	1	/* include all the System V defines */
 # define HASGETUSERSHELL 0	/* does not have getusershell(3) call */
+# define NOFTRUNCATE	0	/* has (simulated) ftruncate call */
 # define MAXPATHLEN	PATHSIZE
-# define LA_TYPE	LA_SHORT
 # define SFS_TYPE	SFS_4ARGS	/* use <sys/statfs.h> 4-arg impl */
 # define SFS_BAVAIL	f_bfree		/* alternate field name */
 # define SPT_TYPE	SPT_SCO		/* write kernel u. area */
 # define TZ_TYPE	TZ_TM_NAME	/* use tm->tm_name */
+# define UID_T		uid_t
+# define GID_T		gid_t
+# define GIDSET_T	gid_t
 # define _PATH_UNIX		"/unix"
 # define _PATH_VENDOR_CF	"/usr/lib/sendmail.cf"
 # ifndef _PATH_SENDMAILPID
@@ -815,9 +850,10 @@ extern int		errno;
 # endif
 
 # ifndef _SCO_DS
-#  define NOFTRUNCATE	0	/* does not have ftruncate(3) call */
+#  define ftruncate	chsize	/* use chsize(2) to emulate ftruncate */
 #  define NEEDFSYNC	1	/* needs the fsync(2) call stub */
 #  define NETUNIX	0	/* no unix domain socket support */
+#  define LA_TYPE	LA_SHORT
 # endif
 
 #endif
@@ -1837,6 +1873,10 @@ extern int	errno;
 # define SIZE_T		size_t
 #endif
 
+#ifndef MODE_T
+# define MODE_T		mode_t
+#endif
+
 #ifndef ARGV_T
 # define ARGV_T		char **
 #endif
