@@ -39,7 +39,7 @@ static char copyright[] =
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)main.c	8.207 (Berkeley) 9/26/96";
+static char sccsid[] = "@(#)main.c	8.215 (Berkeley) 11/16/96";
 #endif /* not lint */
 
 #define	_DEFINE
@@ -137,15 +137,17 @@ main(argc, argv, envp)
 	bool nullserver;
 	char jbuf[MAXHOSTNAMELEN];	/* holds MyHostName */
 	static char rnamebuf[MAXNAME];	/* holds RealUserName */
+	char *emptyenviron[1];
 	extern int DtableSize;
 	extern int optind;
 	extern int opterr;
+	extern char *optarg;
+	extern char **environ;
 	extern time_t convtime();
 	extern void intsig();
 	extern struct hostent *myhostname();
 	extern char *getauthinfo();
 	extern char *getcfname();
-	extern char *optarg;
 	extern void sigusr1();
 	extern void sighup();
 	extern void initmacros __P((ENVELOPE *));
@@ -369,6 +371,11 @@ main(argc, argv, envp)
 	/* initialize for setproctitle */
 	initsetproctitle(argc, argv, envp);
 
+	/* clear sendmail's environment */
+	ExternalEnviron = environ;
+	emptyenviron[0] = NULL;
+	environ = emptyenviron;
+
 	/* prime the child environment */
 	setuserenv("AGENT", "sendmail");
 
@@ -378,14 +385,20 @@ main(argc, argv, envp)
 	(void) setsignal(SIGPIPE, SIG_IGN);
 	OldUmask = umask(022);
 	OpMode = MD_DELIVER;
-	FullName = getenv("NAME");
+	FullName = getextenv("NAME");
+
+	/*
+	**  Initialize name server if it is going to be used.
+	*/
 
 #if NAMED_BIND
-	if (tTd(8, 8))
-	{
+	if (!bitset(RES_INIT, _res.options))
 		res_init();
+	if (tTd(8, 8))
 		_res.options |= RES_DEBUG;
-	}
+# ifdef RES_NOALIASES
+	_res.options |= RES_NOALIASES;
+# endif
 #endif
 
 	errno = 0;
@@ -506,16 +519,7 @@ main(argc, argv, envp)
 			{
 			  case MD_DAEMON:
 			  case MD_FGDAEMON:
-# ifdef DAEMON
-				if (RealUid != 0)
-				{
-					usrerr("Permission denied");
-					exit(EX_USAGE);
-				}
-				(void) unsetenv("HOSTALIASES");
-				vendor_daemon_setup(CurEnv);
-				/* fall through ... */
-# else
+# ifndef DAEMON
 				usrerr("Daemon mode not implemented");
 				ExitStat = EX_USAGE;
 				break;
@@ -526,10 +530,11 @@ main(argc, argv, envp)
 				ExitStat = EX_USAGE;
 				break;
 # endif /* SMTP */
+
+			  case MD_INITALIAS:
 			  case MD_DELIVER:
 			  case MD_VERIFY:
 			  case MD_TEST:
-			  case MD_INITALIAS:
 			  case MD_PRINT:
 			  case MD_HOSTSTAT:
 			  case MD_PURGESTAT:
@@ -649,7 +654,6 @@ main(argc, argv, envp)
 
 		  case 'q':	/* run queue files at intervals */
 # ifdef QUEUE
-			(void) unsetenv("HOSTALIASES");
 			FullName = NULL;
 			queuemode = TRUE;
 			switch (optarg[0])
@@ -792,6 +796,7 @@ main(argc, argv, envp)
 #endif
 	vendor_pre_defaults(CurEnv);
 	readcf(getcfname(), safecf, CurEnv);
+	ConfigFileRead = TRUE;
 	vendor_post_defaults(CurEnv);
 
 	/* avoid denial-of-service attacks */
@@ -835,18 +840,6 @@ main(argc, argv, envp)
 		printf("\n========================================================\n\n");
 	}
 
-	/*
-	**  Initialize name server if it is going to be used.
-	*/
-
-#if NAMED_BIND
-	if (UseNameServer && !bitset(RES_INIT, _res.options))
-		res_init();
-# ifdef RES_NOALIASES
-	_res.options |= RES_NOALIASES;
-# endif
-#endif
-
 	/*
 	**  Do more command line checking -- these are things that
 	**  have to modify the results of reading the config file.
@@ -888,8 +881,8 @@ main(argc, argv, envp)
 	/* check for sane configuration level */
 	if (ConfigLevel > MAXCONFIGLEVEL)
 	{
-		syserr("Warning: .cf version level (%d) exceeds program functionality (%d)",
-			ConfigLevel, MAXCONFIGLEVEL);
+		syserr("Warning: .cf version level (%d) exceeds sendmail version %s functionality (%d)",
+			ConfigLevel, Version, MAXCONFIGLEVEL);
 	}
 
 	/* need MCI cache to have persistence */
@@ -922,6 +915,19 @@ main(argc, argv, envp)
 		/* fall through ... */
 
 	  case MD_DAEMON:
+		/* check for permissions */
+		if (RealUid != 0)
+		{
+#ifdef LOG
+			if (LogLevel > 1)
+				syslog(LOG_ALERT, "user %d attempted to run daemon",
+					RealUid);
+#endif
+			usrerr("Permission denied");
+			exit(EX_USAGE);
+		}
+		vendor_daemon_setup(CurEnv);
+
 		/* remove things that don't make sense in daemon mode */
 		FullName = NULL;
 		GrabTo = FALSE;
@@ -1363,8 +1369,9 @@ main(argc, argv, envp)
 	if (OpMode == MD_VERIFY)
 	{
 		CurEnv->e_sendmode = SM_VERIFY;
-		CurEnv->e_errormode = EM_QUIET;
+		CurEnv->e_errormode = EM_PRINT;
 		PostMasterCopy = NULL;
+		HoldErrs = FALSE;
 	}
 	else
 	{
@@ -1816,6 +1823,31 @@ auth_warning(e, msg, va_alist)
 	}
 }
 /*
+**  GETEXTENV -- get from external environment
+**
+**	Parameters:
+**		envar -- the name of the variable to retrieve
+**
+**	Returns:
+**		The value, if any.
+*/
+
+char *
+getextenv(envar)
+	const char *envar;
+{
+	char **envp;
+	int l;
+
+	l = strlen(envar);
+	for (envp = ExternalEnviron; *envp != NULL; envp++)
+	{
+		if (strncmp(*envp, envar, l) == 0 && (*envp)[l] == '=')
+			return &(*envp)[l + 1];
+	}
+	return NULL;
+}
+/*
 **  SETUSERENV -- set an environment in the propogated environment
 **
 **	Parameters:
@@ -1840,7 +1872,7 @@ setuserenv(envar, value)
 
 	if (value == NULL)
 	{
-		value = getenv(envar);
+		value = getextenv(envar);
 		if (value == NULL)
 			return;
 	}
@@ -1929,6 +1961,15 @@ sighup()
 		syslog(LOG_INFO, "restarting %s on signal", SaveArgv[0]);
 #endif
 	releasesignal(SIGHUP);
+	if (setgid(RealGid) < 0 || setuid(RealUid) < 0)
+	{
+#ifdef LOG
+		if (LogLevel > 0)
+			syslog(LOG_ALERT, "could not set[ug]id(%d, %d): %m",
+				RealUid, RealGid);
+#endif
+		exit(EX_OSERR);
+	}
 	execv(SaveArgv[0], (ARGV_T) SaveArgv);
 #ifdef LOG
 	if (LogLevel > 0)
