@@ -33,7 +33,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)readcf.c	8.173 (Berkeley) 9/20/96";
+static char sccsid[] = "@(#)readcf.c	8.176 (Berkeley) 11/10/96";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -715,7 +715,7 @@ fileclass(class, filename, fmt, safe, optional)
 {
 	FILE *f;
 	int sff;
-	int pid;
+	pid_t pid;
 	register char *p;
 	char buf[MAXLINE];
 
@@ -1024,6 +1024,10 @@ makemailer(line)
 
 				m->m_uid = strtol(p, &q, 0);
 				p = q;
+				while (isascii(*p) && isspace(*p))
+					p++;
+				if (*p != '\0')
+					p++;
 			}
 			while (isascii(*p) && isspace(*p))
 				p++;
@@ -2044,7 +2048,8 @@ setoption(opt, val, safe, sticky, e)
 		goto g_opt;
 
 	  case 'V':		/* fallback MX host */
-		FallBackMX = newstr(val);
+		if (val[0] != '\0')
+			FallBackMX = newstr(val);
 		break;
 
 	  case 'v':		/* run in verbose mode */
