@@ -31,7 +31,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	@(#)sendmail.h	8.200 (Berkeley) 9/26/96
+ *	@(#)sendmail.h	8.209 (Berkeley) 11/8/96
  */
 
 /*
@@ -41,7 +41,7 @@
 # ifdef _DEFINE
 # define EXTERN
 # ifndef lint
-static char SmailSccsId[] =	"@(#)sendmail.h	8.200		9/26/96";
+static char SmailSccsId[] =	"@(#)sendmail.h	8.209		11/8/96";
 # endif
 # else /*  _DEFINE */
 # define EXTERN extern
@@ -187,6 +187,7 @@ typedef struct address ADDRESS;
 
 /* functions */
 extern ADDRESS	*parseaddr __P((char *, ADDRESS *, int, int, char **, ENVELOPE *));
+extern ADDRESS	*buildaddr __P((char **, ADDRESS *, int, ENVELOPE *));
 extern ADDRESS	*recipient __P((ADDRESS *, ADDRESS **, int, ENVELOPE *));
 extern char	**prescan __P((char *, int, char[], int, char **, u_char *));
 extern int	rewrite __P((char **, int, int, ENVELOPE *));
@@ -252,6 +253,7 @@ struct mailer
 # define M_INTERNAL	'I'	/* SMTP to another sendmail site */
 # define M_UDBRECIPIENT	'j'	/* do udbsender rewriting on recipient lines */
 # define M_NOLOOPCHECK	'k'	/* don't check for loops in HELO command */
+# define M_CHUNKING	'K'	/* CHUNKING: reserved for future use */
 # define M_LOCALMAILER	'l'	/* delivery is to this host */
 # define M_LIMITS	'L'	/* must enforce SMTP line limits */
 # define M_MUSER	'm'	/* can handle multiple users at once */
@@ -273,6 +275,7 @@ struct mailer
 # define M_HASPWENT	'w'	/* check for /etc/passwd entry */
 		/*	'x'	   CF: include Full-Name: */
 # define M_XDOT		'X'	/* use hidden-dot algorithm */
+# define M_LMTP		'z'	/* run Local Mail Transport Protocol */
 # define M_NOMX		'0'	/* turn off MX lookups */
 # define M_EBCDIC	'3'	/* extend Q-P encoding for EBCDIC */
 # define M_TRYRULESET5	'5'	/* use ruleset 5 after local aliasing */
@@ -308,7 +311,7 @@ MCI
 	long		mci_maxsize;	/* max size this server will accept */
 	FILE		*mci_in;	/* input side of connection */
 	FILE		*mci_out;	/* output side of connection */
-	int		mci_pid;	/* process id of subordinate proc */
+	pid_t		mci_pid;	/* process id of subordinate proc */
 	char		*mci_phase;	/* SMTP phase string */
 	struct mailer	*mci_mailer;	/* ptr to the mailer for this conn */
 	char		*mci_host;	/* host name */
@@ -731,7 +734,8 @@ extern void	initmaps __P((bool, ENVELOPE *));
 struct symtab
 {
 	char		*s_name;	/* name to be entered */
-	char		s_type;		/* general type (see below) */
+	short		s_type;		/* general type (see below) */
+	short		s_len;		/* length of this entry */
 	struct symtab	*s_next;	/* pointer to next in chain */
 	union
 	{
@@ -1082,6 +1086,7 @@ EXTERN bool	UseNameServer;	/* using DNS -- interpret h_errno & MX RRs */
 EXTERN bool	UseHesiod;	/* using Hesiod -- interpret Hesiod errors */
 EXTERN bool	SevenBitInput;	/* force 7-bit data on input */
 EXTERN bool	HasEightBits;	/* has at least one eight bit input byte */
+EXTERN bool	ConfigFileRead;	/* configuration file has been read */
 EXTERN time_t	SafeAlias;	/* interval to wait until @:@ in alias file */
 EXTERN FILE	*InChannel;	/* input connection */
 EXTERN FILE	*OutChannel;	/* output connection */
@@ -1091,7 +1096,7 @@ EXTERN gid_t	RealGid;	/* real gid of caller */
 EXTERN uid_t	DefUid;		/* default uid to run as */
 EXTERN gid_t	DefGid;		/* default gid to run as */
 EXTERN char	*DefUser;	/* default user to run as (from DefUid) */
-EXTERN int	OldUmask;	/* umask when sendmail starts up */
+EXTERN MODE_T	OldUmask;	/* umask when sendmail starts up */
 EXTERN int	Errors;		/* set if errors (local to single pass) */
 EXTERN int	ExitStat;	/* exit status code */
 EXTERN int	LineNumber;	/* line number in current input */
@@ -1180,6 +1185,7 @@ EXTERN char	*QueueLimitSender;	/* limit queue runs to this sender */
 EXTERN char	*QueueLimitId;		/* limit queue runs to this id */
 EXTERN FILE	*TrafficLogFile;	/* file in which to log all traffic */
 EXTERN char	*DoubleBounceAddr;	/* where to send double bounces */
+EXTERN char	**ExternalEnviron;	/* input environment */
 EXTERN char	*UserEnviron[MAXUSERENVIRON + 1];
 					/* saved user environment */
 extern int	errno;
@@ -1286,6 +1292,7 @@ extern void	logsender __P((ENVELOPE *, char *));
 extern void	smtprset __P((MAILER *, MCI *, ENVELOPE *));
 extern void	smtpquit __P((MAILER *, MCI *, ENVELOPE *));
 extern void	setuserenv __P((const char *, const char *));
+extern char	*getextenv __P((const char *));
 extern void	disconnect __P((int, ENVELOPE *));
 extern void	putxline __P((char *, MCI *, int));
 extern void	dumpfd __P((int, bool, bool));
@@ -1297,7 +1304,7 @@ extern void	inittimeouts __P((char *));
 extern void	logdelivery __P((MAILER *, MCI *, const char *, ADDRESS *, time_t, ENVELOPE *));
 extern void	giveresponse __P((int, MAILER *, MCI *, ADDRESS *, time_t, ENVELOPE *));
 extern void	buildfname __P((char *, char *, char *, int));
-extern void	mci_setstat __P((MCI *, char *, char *));
+extern void	mci_setstat __P((MCI *, int, char *, char *));
 extern char	*smtptodsn __P((int));
 extern int	rscheck __P((char *, char *, char *, ENVELOPE *e));
 extern void	mime7to8 __P((MCI *, HDR *, ENVELOPE *));
@@ -1305,12 +1312,16 @@ extern int	mime8to7 __P((MCI *, HDR *, ENVELOPE *, char **, int));
 extern void	xfclose __P((FILE *, char *, char *));
 extern int	switch_map_find __P((char *, char *[], short []));
 extern void	shorten_hostname __P((char []));
-extern int	waitfor __P((int));
+extern int	waitfor __P((pid_t));
+extern void	proc_list_add __P((pid_t));
+extern void	proc_list_drop __P((pid_t));
 extern void	buffer_errors __P((void));
 extern void	flush_errors __P((bool));
 extern void	putline __P((char *, MCI *));
 extern void	putxline __P((char *, MCI *, int));
 extern bool	xtextok __P((char *));
+extern char	*xtextify __P((char *, char *));
+extern char	*xuntextify __P((char *));
 extern void	cleanstrcpy __P((char *, char *, int));
 extern int	getmxrr __P((char *, char **, bool, int *));
 extern int	strtorwset __P((char *, char **, int));
