@@ -33,7 +33,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)util.c	8.104 (Berkeley) 9/20/96";
+static char sccsid[] = "@(#)util.c	8.109 (Berkeley) 11/16/96";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -1012,6 +1012,15 @@ putxline(l, mci, pxflags)
 			if (TrafficLogFile != NULL)
 				(void) putc('.', TrafficLogFile);
 		}
+		else if (l[0] == 'F' && slop == 0 &&
+			 bitset(PXLF_MAPFROM, pxflags) &&
+			 strncmp(l, "From ", 5) == 0 &&
+			 bitnset(M_ESCFROM, mci->mci_mailer->m_flags))
+		{
+			(void) putc('>', mci->mci_out);
+			if (TrafficLogFile != NULL)
+				(void) putc('>', TrafficLogFile);
+		}
 		if (TrafficLogFile != NULL)
 			fprintf(TrafficLogFile, "%.*s\n", p - l, l);
 		for ( ; l < p; ++l)
@@ -1363,19 +1372,21 @@ atooct(s)
 
 int
 waitfor(pid)
-	int pid;
+	pid_t pid;
 {
 #ifdef WAITUNION
 	union wait st;
 #else
 	auto int st;
 #endif
-	int i;
+	pid_t i;
 
 	do
 	{
 		errno = 0;
 		i = wait(&st);
+		if (i > 0)
+			proc_list_drop(i);
 	} while ((i >= 0 || errno == EINTR) && i != pid);
 	if (i < 0)
 		return -1;
@@ -2172,3 +2183,125 @@ path_is_dir(pathname, createflag)
 	}
 	return TRUE;
 }
+/*
+**  PROC_LIST_ADD -- add process id to list of our children
+**
+**	Parameters:
+**		pid -- pid to add to list.
+**
+**	Returns:
+**		none
+*/
+
+static pid_t	*ProcListVec	= NULL;
+static int	ProcListSize	= 0;
+
+#define NO_PID		((pid_t) 0)
+#ifndef PROC_LIST_SEG
+# define PROC_LIST_SEG	32		/* number of pids to alloc at a time */
+#endif
+
+void
+proc_list_add(pid)
+	pid_t pid;
+{
+	int i;
+	extern void proc_list_probe __P((void));
+
+	for (i = 0; i < ProcListSize; i++)
+	{
+		if (ProcListVec[i] == NO_PID)
+			break;
+	}
+	if (i >= ProcListSize)
+	{
+		/* probe the existing vector to avoid growing infinitely */
+		proc_list_probe();
+
+		/* now scan again */
+		for (i = 0; i < ProcListSize; i++)
+		{
+			if (ProcListVec[i] == NO_PID)
+				break;
+		}
+	}
+	if (i >= ProcListSize)
+	{
+		/* grow process list */
+		pid_t *npv;
+
+		npv = (pid_t *) xalloc(sizeof (pid_t) * (ProcListSize + PROC_LIST_SEG));
+		if (ProcListSize > 0)
+		{
+			bcopy(ProcListVec, npv, ProcListSize * sizeof (pid_t));
+			free(ProcListVec);
+		}
+		for (i = ProcListSize; i < ProcListSize + PROC_LIST_SEG; i++)
+			npv[i] = NO_PID;
+		i = ProcListSize;
+		ProcListSize += PROC_LIST_SEG;
+		ProcListVec = npv;
+	}
+	ProcListVec[i] = pid;
+	CurChildren++;
+}
+/*
+**  PROC_LIST_DROP -- drop pid from process list
+**
+**	Parameters:
+**		pid -- pid to drop
+**
+**	Returns:
+**		none.
+*/
+
+void
+proc_list_drop(pid)
+	pid_t pid;
+{
+	int i;
+
+	for (i = 0; i < ProcListSize; i++)
+	{
+		if (ProcListVec[i] == pid)
+		{
+			ProcListVec[i] = NO_PID;
+			break;
+		}
+	}
+	if (CurChildren > 0)
+		CurChildren--;
+}
+/*
+**  PROC_LIST_PROBE -- probe processes in the list to see if they still exist
+**
+**	Parameters:
+**		none
+**
+**	Returns:
+**		none
+*/
+
+void
+proc_list_probe()
+{
+	int i;
+
+	for (i = 0; i < ProcListSize; i++)
+	{
+		if (ProcListVec[i] == NO_PID)
+			continue;
+		if (kill(ProcListVec[i], 0) < 0)
+		{
+#ifdef LOG
+			if (LogLevel > 3)
+				syslog(LOG_DEBUG, "proc_list_probe: lost pid %d",
+					ProcListVec[i]);
+#endif
+			ProcListVec[i] = NO_PID;
+			CurChildren--;
+		}
+	}
+	if (CurChildren < 0)
+		CurChildren = 0;
+}
